# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import typing
from .client_side_access_tokens_create_client_side_access_token_response_policy_data_item import (
    ClientSideAccessTokensCreateClientSideAccessTokenResponsePolicyDataItem,
)
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class ClientSideAccessTokensCreateClientSideAccessTokenResponsePolicy(UncheckedBaseModel):
    version: typing.Literal["1"] = "1"
    data: typing.List[ClientSideAccessTokensCreateClientSideAccessTokenResponsePolicyDataItem]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
