# This file was auto-generated by Fern from our API Definition.

import typing
from ...types.completion_create_params_non_streaming import CompletionCreateParamsNonStreaming
from ...types.completion_create_params_streaming import CompletionCreateParamsStreaming

CreateVoiceChatCompletionsRequestBody = typing.Union[
    CompletionCreateParamsNonStreaming, CompletionCreateParamsStreaming
]
