# This file was auto-generated by Fern from our API Definition.

from ..core.api_error import ApiError
from ..types.conflict_error_body import ConflictErrorBody


class ConflictError(ApiError):
    def __init__(self, body: ConflictErrorBody):
        super().__init__(status_code=409, body=body)
