# This file was auto-generated by Fern from our API Definition.

import typing

ClientSideAccessTokensCreateClientSideAccessTokenRequestPolicyItemAccessItem = typing.Union[
    typing.Literal["read_messages", "write_messages", "read_agent", "write_agent"], typing.Any
]
